# 项目：标准函数库
# 模块：业务测试模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2017-01-04 21:34

# Remember:
# Do not modify this file!!!
# 

import sys
sys.path.insert(0,'.')
from testing import *
unittest.main()

